/*
   SETPROMP.C:
      Generates random colors and attributes for the DOS command prompt.
      After generating the colors it makes sure that the foreground and
      background colors are not the same then modifies the variable
      "PROMPT" in the master DOS environment.

      This is an adaptation of SETENV.C

      It is required to have ANSI.SYS loaded from the CONFIG.SYS file.

      This is my first real "C" program so please excuse the poor
      programming style.

      Written in Turbo C version 2.0.

      Author: Brad Larsen      CIS ID#: 74504,721     DATE: 11-11-94
              Park Falls, WI
*/

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <alloc.h>
#include <dos.h>
#include <string.h>
#include "proto.h"


/*
   This will set the PROMPT to the path followed by the greater than sign.
   Modify this line if you want something different.

   The colors of the foreground and background will be changed randomally
   before this string is written. Then the colors will changed back to the
   default foreground and background colors that are also set below.
*/

char           prompt_string[256] = "$p$g";
char           default_attribute_string[4] = "1";
char           default_foreground_string[4] = "37";
char           default_background_string[4] = "40";


/*
   Colors

      BLACK_FOREGROUND     = 30
      RED_FOREGROUND       = 31
      GREEN_FOREGROUND     = 32
      YELLOW_FOREGROUND    = 33
      BLUE_FOREGROUND      = 34
      MAGENTA_FOREGROUND   = 35
      CYAN_FOREGROUND      = 36
      WHITE_FOREGROUND     = 37

      BLACK_BACKGROUND     = 40
      RED_BACKGROUND       = 41
      GREEN_BACKGROUND     = 42
      YELLOW_BACKGROUND    = 43
      BLUE_BACKGROUND      = 44
      MAGENTA_BACKGROUND   = 45
      CYAN_BACKGROUND      = 46
      WHITE_BACKGROUND     = 47

      NORMAL               = 0
      BOLD                 = 1
      BLINK                = 5
      CONCEAL              = 8

   Other character combinations that can be used for the PROMPT variable

      $Q                     = (equal sign
      $$                     $ (dollar sign)
      $T                     Current time
      $D                     Current data
      $P                     Current drive and path
      $V                     MS-DOS version number
      $N                     Current drive
      $G                     > (greater-than sign)
      $L                     < (less-than sign)
      $B                     | (pipe)
      $_                     ENTER_LINEFEED
      $E                     ASCII escape code (code 27)
      $H                     Backspace (to delete a character that has been
                                written to the prompt command line)
*/


void main(int argc, char *argv[])
{
   char      far  *env_str_address;
   char      far  *env_str_ptr;
   unsigned       max_env_size;
   unsigned       cur_env_size;
   char           name[8];
   char           value[256];
   char           new_variable[256];
   char           temp_string[85];
   unsigned       state = CHECK_PARAMETERS;
   unsigned       done = FALSE;
   unsigned       ansi_loaded = FALSE;
   unsigned       enable_blink = TRUE;
   unsigned       enable_bold = FALSE;
   unsigned       enable_ansi_check = TRUE;
   int            att = 0;
   int            attribute_limit = 2;
   int            foreground = 0;
   int            background = 0;
   int            x = 0;
   char           att_string[4];
   char           foreground_string[4];
   char           background_string[4];
   FILE           *temp_file;

   while(!done)
   {
      switch(state)
      {
         case CHECK_PARAMETERS:
            switch(argc)
            {
               case 1:
                  /* There are no extra command line parameters */
                  /* Allow blinking attribute */
                  attribute_limit = 6;
                  state = CHECK_FOR_ANSI;
                  break;

               default:
                  /* There at least one extra command line parameters */
                  for(x = 1; x < argc; x++)
                  {
                     /* If it is /HELP display the help screen */
                     if(!stricmp(argv[x], "/HELP"))
                     {
                        display_help();
                        x = argc;
                        state = FINISHED;
                     }
                     /* If it is /? display the help screen */
                     else if(!stricmp(argv[x], "/?"))
                     {
                        display_help();
                        x = argc;
                        state = FINISHED;
                     }
                     /* If it is /NOBLINK don't allow the blink attribute */
                     else if(!stricmp(argv[x], "/NOBLINK"))
                     {
                        enable_blink = FALSE;
                        state = CHECK_FOR_ANSI;
                     }
                     /* If it is /BOLD force the bold attribute */
                     else if(!stricmp(argv[x], "/BOLD"))
                     {
                        enable_bold = TRUE;
                        state = CHECK_FOR_ANSI;
                     }
                     /* If it is /NOANSICHECK skip check for ANSI.SYS loaded */
                     else if(!stricmp(argv[x], "/NOANSICHECK"))
                     {
                        enable_ansi_check = FALSE;
                        state = CHECK_FOR_ANSI;
                     }
                     /* If it isn't any of the above display help */
                     else
                     {
                        printf("\n\nIllegal Parameter\n\n");
                        display_help();
                        x = argc;
                        state = FINISHED;
                     }
                  }
                  break;
            }
            break;

         case CHECK_FOR_ANSI:
            /* Check if ANSI.SYS is loaded */
            /* Sleazy but it works. Redirect the mem /c command to a file */
            /* Parse the file to see if the string "ANSI" is in the ile */
            if(enable_ansi_check)
            {
               system("mem /c > \\ttt.tmp");
               temp_file = fopen("\\ttt.tmp", "rt");
               if(temp_file != NULL)
               {
                  while(!feof(temp_file))
                  {
                     fgets(temp_string, 80, temp_file);
                     if(strstr(temp_string, "ANSI") != NULL) /* ANSI loaded */
                     {
                        ansi_loaded = TRUE;
                        state = INITIALIZE;
                     }
                  }
                  if(!ansi_loaded) /* ANSI not loaded */
                  {
                     printf("\nERROR: ANSI.SYS not loaded\n");
                     display_help();
                     state = FINISHED;
                  }
                  /* delete temporary file */
                  remove("\\ttt.tmp");
               }
               else /* Problem accessing temporary file */
               {
                  printf("\nTemporary file could not be accessed.\n");
                  state = FINISHED;
               }
            }
            else
            {
               printf("\nSkipping ANSI.SYS check\n");
               state = INITIALIZE;
            }
            break;

         case INITIALIZE:
            /* seed random number generator */
            randomize();

            /* Determine normal, bright or blink attribute.                   */
            /* Restrict random number generator to 2 if you do not wish blink */
            if(enable_bold)
            {
               printf("\nBold foreground prompt forced\n");
               att = 1;
            }
            else if(!enable_blink)
            {
               printf("\nBlinking prompt disabled\n");
               attribute_limit = 2;
            }
            else
            {
               att = random(attribute_limit);
            }

            /* Generate foreground color */
            while(foreground < 30)
            {
               foreground = random(38);
            }

            /* Generate background color */
            while(background < 40)
            {
               background = random(48);
            }

            /* Make sure foreground and background are not the same. */
            /* If they are set the prompt to white on black. */
            if(foreground == (background - 10))
            {
               if(enable_bold)
               {
                  att = 1;
               }
               else
               {
                  att = 0;
               }
               foreground = 37;
               background = 40;
            }

            /* Convert integers to strings. */
            itoa(att, att_string, 10);
            itoa(foreground, foreground_string, 10);
            itoa(background, background_string, 10);

            /* Create new prompt string using ANSI escape codes. */
            sprintf(name, "PROMPT=");
            sprintf(new_variable, "%s$e[%s;%s;%sm%s$e[%s;%s;%sm", name,
                    att_string, foreground_string, background_string,
                    prompt_string, default_attribute_string,
                    default_foreground_string, default_background_string);

            state = GET_ENV_INFO;
            break;

         case GET_ENV_INFO:
            /* Get enviromnent information */
            get_env_info(&env_str_address, &max_env_size, &cur_env_size);
            state = GET_VARIABLE_POINTER;
            break;

         case GET_VARIABLE_POINTER:
            /* Get pointer to the beginning of the PROMPT variable */
            env_str_ptr = get_name_ptr(env_str_address, name, value);
            state = CHECK_FOR_ROOM;
            break;

         case DELETE_VARIABLE:
            /* Delete current PROMPT variable */
            del_variable(env_str_address, env_str_ptr, &cur_env_size);
            state = APPEND_VARIABLE;
            break;

         case CHECK_FOR_ROOM:
            if(env_str_ptr == NULL) /* Variable doesn't exist */
            {
               if(check_for_room(max_env_size, cur_env_size, new_variable, 0))
               {
                  state = APPEND_VARIABLE;
               }
               else
               {
                  state = NOT_ENOUGH_ROOM;
               }
            }
            else /* Variable is in environment */
            {
               if(check_for_room(max_env_size, cur_env_size, new_variable,
                                 strlen(name) + strlen(value) + 1))
               {
                  state = DELETE_VARIABLE;
               }
               else
               {
                  state = NOT_ENOUGH_ROOM;
               }
            }
            break;

         case NOT_ENOUGH_ROOM:
            printf("\nNot enough room for new variable.\n");
            state = FINISHED;
            break;

         case APPEND_VARIABLE:
            /* put the PROMPT at the end of the environment */
            append_new_variable(env_str_address, new_variable);
            state = FINISHED;
            break;

         case FINISHED:
            /* Exit program */
            done = TRUE;
            break;
      }
   }
}



/*
   get_env_info:
      This routine returns a pointer to the beginning of the master DOS
      environment, the max size as set in CONFIG.SYS, and the current size.
*/
void get_env_info(char far **env_str_address, unsigned *max_env_size, unsigned *cur_env_size)
{
   unsigned  far  *env_address;
   unsigned  far  *env_size_ptr;
   unsigned  far  *env_seg_ptr;
   unsigned  far  *first_pointer;
   unsigned       first_null;
   unsigned       counter = 0;
   char      far  *ptr;

   int            vector = 0x002E * 4;      /* 0x0000:0x00B8                 */
   int            offset = 0x002C;          /* offset to env segment pointer */

   first_pointer = MK_FP(0x0000, vector + 2);
   env_seg_ptr = MK_FP(*first_pointer, offset);

   /* address of start of environment */
   env_address = MK_FP(*env_seg_ptr, 0);

   /* Create index pointer to be used later                             */
   ptr = (char far *) env_address;

   /* Cast as a character pointer                                       */
   *env_str_address = (char far *)MK_FP(*env_seg_ptr, 0);

   /* 1 paragraph back at offset 3 is max environment size in 16 byte   */
   /* paragraphs.                                                       */
   env_size_ptr = MK_FP((*env_seg_ptr - 1), 0x0003);
   *max_env_size = 16 * (*env_size_ptr);

   /* determine current environment size. It ends with 2 NULL's.        */
   for(; !((*ptr == NULL) && (*(ptr - 1) == NULL)) ; ptr++, counter++);
   *cur_env_size = counter;
}



/*
   get_name_ptr:
      Scans the master environment for a given string and returns a
      pointer to it. Else returns NULL. It also returns the value of the
      given variable if it exist. If it does not, value is set to a NULL
      string.
*/
char far *get_name_ptr (char far *env_str_address , char *name, char *value)
{
   char far *index;
   char far *index1;
   char far *str1;
   char far *str2;
   unsigned counter;

   /*  Set index to beginning of environment */
   index = env_str_address;

   /*  While not at the end of the environment */
   while(*index)
   {
      for ( str1 = index , str2 = name; (*str1) && (*str2) && (*str1 == *str2)
            ;str1++ , str2++ ) ;

      /* Reached the end of the string.                                 */
      /* Return a pointer start of this string                          */
      if (!*str2)                     /* reached the end of name string */
      {
         index1 = index;
         for(; *index1 != '='; index1++);
         index1++;
         /* Get the value of the existing variable                      */
         for(counter = 0 ; *index1 != NULL ;)
         {
            value[counter] = *index1;
            index1++;
            counter++;
         }
         value[counter] = NULL;
         return (index);
      }

      /*  Advance to the next variable in the env.                      */
      for ( ; *(index++) ; ) ;
   }

   /* String is not present.                                            */
  return ( NULL ) ;
}




/*
   check_for_room:
      Returns TRUE if there is room for new variable plus a
      terminating NULL. Else returns FALSE. Takes into account if a variable
      is to be deleted.
*/
unsigned check_for_room(unsigned max_env_size, unsigned cur_env_size, char *new_variable, unsigned size_to_delete)
{
   unsigned  ret = FALSE;

   if(max_env_size > (cur_env_size + strlen(new_variable) - size_to_delete + 1))
   {
      ret = TRUE;
   }
   return(ret);
}



/*
   del_variable:
      Locates the beginning of the string to be overwritten. Advances a
      pointer to the next string and writes the remainder of environment over
      the top of the variable to be deleted
*/
void del_variable(char far *env_str_address, char far *env_str_ptr, unsigned *cur_env_size)
{
   char     far  *index;
   unsigned      done = FALSE;

   /* Go to beginning of string to be deleted                           */
   while(!(env_str_address == env_str_ptr))
   {
      env_str_address = env_str_address + 1;
   }
   /* Mark this place                                                   */
   index = env_str_address;

   /* Move to the beginning of the next string adjusting the current    */
   /* environment size as you go.                                       */
   while(!(*env_str_address == NULL))
   {
      env_str_address++;
      *cur_env_size = *cur_env_size - 1;
   }
   /* Final adjustments                                                 */
   env_str_address++;
   *cur_env_size = *cur_env_size - 1;

   /* Overwrite variable to be deleted till at end of environment       */
   while(!done)
   {
      *index = *env_str_address;
      if((*env_str_address == NULL) && (*(env_str_address - 1) == NULL))
      {
         done = TRUE;
      }
      env_str_address = env_str_address + 1;
      index = index + 1;
   }
}




/*
   append_new_variable:
      Go to the end of the environment and add the new variable, then
      add the last terminating NULL's
*/
void append_new_variable(char far *env_str_address, char *new_variable)
{
   char       *new_variable_ptr;

   new_variable_ptr = new_variable;                 /* create a pointer */

   /* Advance to end of environment                                     */
   for(;!((*env_str_address == NULL) && (*(env_str_address - 1) == NULL));)
   {
      env_str_address++;
   }

   for(; *new_variable_ptr ;)
   {
      *env_str_address = *new_variable_ptr;
      env_str_address++;
      new_variable_ptr++;
   }
   /* End environment with two NULL's                                   */
   *env_str_address = NULL;
   *(env_str_address + 1) = NULL;
}



/*
   DISPLAY_HELP:
      Display help screen.
*/
void display_help(void)
{
   printf("\nThis utility generates random prompt colors.\n");
   printf("\nSyntax:  SETPROMP [/HELP | /?] [/NOBLINK] [/BOLD]\n");
   printf("\n");
   printf("/NOBLINK     will not allow blinking prompts.\n");
   printf("/BOLD        will force the bold foreground attribute and also ");
   printf("disable blinking.\n");
   printf("/NOANSICHECK will skip the check for ANSI.SYS loaded. Usefull\n");
   printf("             if your memory manager doesn't display the names of\n");
   printf("             drivers loaded high when using the MEM /C command,\n");
   printf("             as with 386MAX.\n");
   printf("\n/? or /HELP displays this screen.\n");
   printf("\nANSI.SYS must be loaded from the CONFIG.SYS file unless the\n");
   printf("/NOANSICHECK parameter is used.\n");
}
